/*------------------------------------------------------------------------*/
/* FILE NAME: WR_MPC5566_rom.ld             COPYRIGHT (c) FREESCALE 2006  */
/*                                                                        */
/* DESCRIPTION:                                                           */
/*    MPC5566 Example linker File for the WindRiver compiler              */
/* ORIGINAL AUTHOR: Chris Baker                                           */
/* REV      AUTHOR       DATE       DESCRIPTION OF CHANGE                 */
/* ---   -----------   ----------   ---------------------                 */
/* 0.1   Chris Baker   30/Aug/06     Initial version.                     */
/*------------------------------------------------------------------------*/


MEMORY
{
/* 2M External Flash (unused) */
/* 512K External SRAM (unused) */
        ext_flash : org = 0x20000000, len = 0x00200000
        ext_ram   : org = 0x3ff80000, len = 0x80000

/* Internal Flash RCW */
/* MPC5566 3M Internal Flash, but subtract one 128K block for use by BAM. */
/* MPC5566 128K Internal SRAM */
/* MPC5566 4K of internal cache used for stack. */
        flash_rcw : org = 0x00000000, len = 0x8
        int_flash : org = 0x00000008, len = 0x002FFFF8
        int_sram  : org = 0x40000000, len = 0x20000
        stack_ram : org = 0x40040000, len = 0x1000
}


SECTIONS
{

/* ROM data */
    .rcw  : { *(.rcw) } > flash_rcw

    GROUP :
    {
	.init       : { *(.init) } 
	.text       : {}
	.flash_data : {}
	.rodata  : 
	{
         *(.rdata) 
         *(.rodata) 
    }

	.= ((. + 7) & ~7); /* align __DATA_ROM to .data start */
         __DATA_ROM =.;
         . = . + SIZEOF(.data); /* make space for .data */
         __SDATA_ROM = .;
         . = . + SIZEOF(.sdata); /* make space for .sdata */
         __DATA_ROM_END = .;  /* end of __DATA_ROM */

	.ctors : {}
	.dtors : {}

	/*the interrupt vector table for the INTC must be aligned on 2K boundary*/
	.isrvectbl ALIGN(0x800): {}

	.xcptn: {}  
	extabindex : {}
     } > int_flash



    /* SRAM data */
    GROUP :
    {
	.data   LOAD(__DATA_ROM): {}
	.sdata  LOAD(__SDATA_ROM): {}
	.sbss   : {}
	.sdata2 : {}
	.sbss2  : {}
	.bss    : {}
	.heap   : {}
    } > int_sram

}

/* Stack Address Parameters */
__SP_INIT      = ADDR(stack_ram) + SIZEOF(stack_ram); 
__SP_END       = ADDR(stack_ram); 
__STACK_SIZE   = SIZEOF(stack_ram); 


/* SRAM Address Parameters */
__SRAM_CPY_START = ADDR(.data);
__ROM_COPY_SIZE  = (SIZEOF(.data) + SIZEOF(.sdata));
__SRAM_LOAD      = ADDR(.heap);
__SRAM_LOAD_SIZE = (SIZEOF(.flash_data)/4);

/* Interrupt Handler Parameters */
__IV_ADDR     = ADDR(.xcptn);